#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <errno.h>

#define BUILD_LIBRARY

#include "ethernet.h"
#include "ADEthernet.h"
#include "libadwin/errno.h"
#include "libadwin/types.h"
#include "libadwin/bits.h"
// todo
#include "libadwin_undocumented.h"

long NET_Boot(adwin_dev_t *dev, int fd, const char *filename, long iboardsize)
{
    Generic_Driver_Boot(filename);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Test_Version(adwin_dev_t *dev, int fd)
{
    return Generic_Driver_Test_Version();
    //adwin_errno = Generic_Driver_Get_Last_Error();
    //return adwin_errno ? -1 : 0;
}

processor_type_t NET_Processor_Type(adwin_dev_t *dev, int fd, processor_type_t *processor)
{
    *processor = Generic_Driver_Processor_Type();
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;

}

int NET_Workload(adwin_dev_t *dev, int fd, int32_t priority, int32_t *value)
{
    *value = Generic_Driver_Workload(priority);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Free_Mem(adwin_dev_t *dev, int fd, int mem_spec, int32_t *value)
{
    *value = Generic_Driver_Free_Mem(mem_spec);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Load_Process(adwin_dev_t *dev, int fd, const char *filename)
{
    Generic_Driver_Load_Process(filename);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Start_Process(adwin_dev_t *dev, int fd, int process_no)
{
    // mapping to be compatible with transputer, etc.
    Generic_Driver_Start_Process((process_no - 99) / 2);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Stop_Process(adwin_dev_t *dev, int fd, int process_no)
{
    // mapping to be compatible with transputer, etc.
    Generic_Driver_Stop_Process((process_no - 99) / 2);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Clear_Process(adwin_dev_t *dev, int fd, int process_no)
{
    Generic_Driver_Clear_Process(process_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Set_Par(adwin_dev_t *dev, int fd, int index, int32_t value)
{
    Generic_Driver_Set_Par(index - 1000, value);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Set_FPar(adwin_dev_t *dev, int fd, int index, float value)
{
    Generic_Driver_Set_FPar(index - 1100, value);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Set_FPar_Double(adwin_dev_t *dev, int fd, int index, double value)
{
    Generic_Driver_Set_FPar_Double(index - 1200, value);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Get_Par(adwin_dev_t *dev, int fd, int index, int32_t *value)
{
    *value = Generic_Driver_Get_Par(index - 1000);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Get_FPar(adwin_dev_t *dev, int fd, int index, float *value)
{
    *value = Generic_Driver_Get_FPar(index - 1100);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Get_FPar_Double(adwin_dev_t *dev, int fd, int index, double *value)
{
    *value = Generic_Driver_Get_FPar_Double(index - 1200);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Get_Par_All(adwin_dev_t *dev, int fd, int start_index, int count, int32_t *dest)
{
    Generic_Driver_Get_Par_Block(dest, start_index, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Get_FPar_All(adwin_dev_t *dev, int fd, int start_index, int count, float *dest)
{
    Generic_Driver_Get_FPar_Block(dest, start_index - 100, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Get_FPar_All_Double(adwin_dev_t *dev, int fd, int start_index, int count, double *dest)
{
    Generic_Driver_Get_FPar_Block_Double(dest, start_index - 200, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}


int NET_GetData(adwin_dev_t *dev, int fd, int data_no, long start_index, long count, void *dest, int16_t type)
{
    Generic_Driver_Get_Data(data_no, dest, type, start_index, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

long NET_Data_Length(adwin_dev_t *dev, int fd, long data_no)
{
    int32_t tmp = Generic_Driver_Data_Length(data_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : tmp;
}

int NET_GetData_String(adwin_dev_t *dev, int fd, int data_no, long max_count, void *dest)
{
    Generic_Driver_GetData_String(data_no, dest, max_count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_SetData_String(adwin_dev_t *dev, int fd, int data_no, long count, const void *src)
{
    Generic_Driver_SetData_String(data_no, src);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_String_Length(adwin_dev_t *dev, int fd, int data_no, int *dest)
{
    *dest = Generic_Driver_String_Length(data_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}


int NET_SetData(adwin_dev_t *dev, int fd, int data_no, long start_index, long count, void *src, int16_t type)
{
    Generic_Driver_Set_Data(data_no, src, type, start_index, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Fifo_Empty(adwin_dev_t *dev, int fd, int fifo_no, int *dest)
{
    *dest = Generic_Driver_Fifo_Empty(fifo_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Fifo_Full(adwin_dev_t *dev, int fd, int fifo_no, int *dest)
{
    *dest = Generic_Driver_Fifo_Full(fifo_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_Fifo_Clear(adwin_dev_t *dev, int fd, int fifo_no)
{
    Generic_Driver_Fifo_Clear(fifo_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_SetFifo(adwin_dev_t *dev, int fd, int fifo_no, long count, void *src, int16_t type)
{
    Generic_Driver_Set_Fifo(fifo_no, type, src, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

int NET_GetFifo(adwin_dev_t *dev, int fd, int fifo_no, long count, void *dest, int16_t type)
{
    Generic_Driver_Get_Fifo(fifo_no, type, dest, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

// ************************************************************************************
// ***************************** unsupported ******************************************

int NET_GetDataType(adwin_dev_t *dev, int fd, int data_no)
{
    int tmp;
    tmp = Generic_Driver_GetDataType(data_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : tmp;
}

int NET_Clear_Data(adwin_dev_t *dev, int fd, int Datanr)
{
    Generic_Driver_Clear_Data(Datanr);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}

/* short NET_CreateData(adwin_dev_t *dev, int fd, short datanr, short type, long groesse, short ort, short art) { */
/* } */

int NET_Get_Memory_dsp(adwin_dev_t *dev, int fd, int32_t base_addr, int32_t count, int32_t *dest)
{
    Generic_Driver_Get_Memory_dsp(base_addr, count, dest);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : 0;
}
